/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

package com.lucsorel.hesperis.client;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.lucsorel.hesperis.client.modele.Erreur;
import com.lucsorel.hesperis.client.modele.Warning;
import com.lucsorel.hesperis.client.modele.Message;
import com.lucsorel.hesperis.client.modele.Utilisateur;

class Connexion
{
	// lien vers l'application principale
	final Hesperis iH_hesperis;

	// widgets
	final HorizontalPanel iHP_connexionPanel;
	final HTML iH_message;
	final TextBox iTB_login;
	final PasswordTextBox iPTB_motDePasse;
	final Button iB_connexion;

	public Connexion( Hesperis pH_hesperis )
	{
		// mémorisation du lien à l'application principale
		iH_hesperis = pH_hesperis;

		/*
		 *  Création du widget
		 */
		// 1. création du panneau général de la vue
		iHP_connexionPanel = new HorizontalPanel();
		iHP_connexionPanel.setStylePrimaryName( "connexion-global" );

		// 2. création du panneau contenant les champs de connexion
		VerticalPanel lVP_loginPanel = new VerticalPanel();
		lVP_loginPanel.setStylePrimaryName( "login" );

		// 2.1 compte
		HorizontalPanel lHP_loginCompte = new HorizontalPanel();
		lHP_loginCompte.add( new Label("Compte : ") );
		iTB_login = new TextBox();
		// simule un clic sur le bouton de connexion si la touche [Entrée] est appuyée dans ce champ
		iTB_login.addKeyUpHandler( new ConnexionPressEnterHandler() );
		lHP_loginCompte.add( iTB_login );

		// 2.2 mot de passe
		HorizontalPanel lHP_loginMotDePasse = new HorizontalPanel();
		lHP_loginMotDePasse.add( new Label("Mot de passe : ") );
		iPTB_motDePasse = new PasswordTextBox();
		// simule un clic sur le bouton de connexion si la touche [Entrée] est appuyée dans ce champ
		iPTB_motDePasse.addKeyUpHandler( new ConnexionPressEnterHandler() );
		lHP_loginMotDePasse.add( iPTB_motDePasse );

		// 2.3 bouton de connexion
		iB_connexion = new Button( "Me connecter" );
		iB_connexion.addClickHandler( new ConnexionClickHandler() );

		// 3. panneau d'affichage de messages
		iH_message = new HTML();
		iH_message.setStylePrimaryName( "message" );

		// 4. assemblage de l'interface
		lVP_loginPanel.add( lHP_loginCompte );
		lVP_loginPanel.add( lHP_loginMotDePasse );
		lVP_loginPanel.add( iB_connexion );
		iHP_connexionPanel.add( lVP_loginPanel );
		iHP_connexionPanel.add( iH_message );
	}

	public Widget asWidget()
	{
		return iHP_connexionPanel;
	}

	/**
	 * Simule un clic de demande de connexion lorsqu'on appuie sur [Entrée]
	 */
	class ConnexionPressEnterHandler implements KeyUpHandler
	{
		@Override
		public void onKeyUp(KeyUpEvent pE_keyUp )
		{
			if( pE_keyUp.getNativeKeyCode() == KeyCodes.KEY_ENTER )
			{
				new ConnexionClickHandler().onClick( null );
			}
		}
	}

	/**
	 * Gère une demande de connexion par une requête HTTP au serveur. Ce gestionnaire est sollicité par :
	 * - un clic réel sur le bouton
	 * - une simulation par PressEnterConnexionHandler
	 */
	class ConnexionClickHandler implements ClickHandler
	{
		@Override
		public void onClick( ClickEvent pE_clicConnexion )
		{
			String ls_demandeConnexion = "controleur.php?"
				+ "action=connecter&login=" + iTB_login.getText()
				+ "&password=" + iPTB_motDePasse.getText();
			ls_demandeConnexion = URL.encode(ls_demandeConnexion);

			// envoi de la requête avec la méthode POST
			RequestBuilder lRB_requeteDemandeConnexion = new RequestBuilder( RequestBuilder.POST, ls_demandeConnexion );
			try
			{
				lRB_requeteDemandeConnexion.sendRequest( null, new ConnexionRequestCallback() );
			}
			catch( RequestException pRE_exception )
			{
				afficherMessage( new Erreur(
						"Requête de connexion erronnée :\n" + pRE_exception.getMessage()) );
			}
		}
	}

	/**
	 * Gère le retour de la requête de déconnexion
	 */
	class ConnexionRequestCallback implements RequestCallback
	{
		@Override
		public void onError( Request request, Throwable pRE_exception )
		{
			afficherMessage( new Erreur(
					"Requête de connexion erronnée :\n" + pRE_exception.getMessage()) );
		}
		@Override
		public void onResponseReceived( Request request, Response pR_retourConnexion )
		{
			// gestion des erreurs
			String ls_texteReponse = pR_retourConnexion.getText();
			Message lM_message = null;
			Utilisateur lU_utilisateur = null;
			if( pR_retourConnexion.getStatusCode() != Response.SC_OK )
			{
				lM_message = new Erreur( "Erreur : " + pR_retourConnexion.getStatusText()
						+ "\n Code : " + pR_retourConnexion.getStatusCode()
						+ "\n Header : [" + pR_retourConnexion.getHeadersAsString() + "]" );
			}
			// récupération et traitement des données utilisateur
			else
			{
				lU_utilisateur = Utilisateur.Instance();
				lU_utilisateur.enrichirSelonJson( ls_texteReponse );
				if( ! lU_utilisateur.estValide() )
				{
					// création du message d'erreur en interprétant le retour de requête comme du JSON ou tel quel
					lM_message = Erreur.ErreurSelonJSON( ls_texteReponse );
					if( lM_message == null )
					{
						lM_message = new Warning( ls_texteReponse );
					}
				}
				else
				{
					lM_message = new Message( "Bienvenue <i>" + lU_utilisateur.getNom() + "</i>, vous allez être <b>connecté(e)</b> à Hespéris ! " );
				}
			}

			afficherMessage( lM_message );

			if( lU_utilisateur.estValide() )
			{
				// Affichage de la déconnexion
				iH_hesperis.afficherDeconnexion( true );
			}
		}
	}

	/**
	 * Affiche un message dans le panneau de message
	 * @param pM_message 
	 */
	public void afficherMessage( Message pM_message )
	{
		if( pM_message instanceof Erreur )
		{
			iH_message.setStylePrimaryName( "message-erreur" );
			iH_message.setHTML( pM_message.getMessage() );
		}
		else if( pM_message instanceof Warning )
		{
			iH_message.setStylePrimaryName( "message-warning" );
			iH_message.setHTML( pM_message.getMessage() );
		}
		// nettoyage du stylage, affichage du message
		else if( pM_message != null )
		{
			iH_message.setStylePrimaryName( "message" );
			iH_message.setHTML( pM_message.getMessage() );
		}
		// nettoyage du stylage et du message
		else
		{
			iH_message.setStylePrimaryName( "message" );
			iH_message.setHTML( "" );
		}
	}
}