/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

package com.lucsorel.hesperis.client;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.Response;
import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.lucsorel.hesperis.client.modele.Erreur;
import com.lucsorel.hesperis.client.modele.Message;
import com.lucsorel.hesperis.client.modele.Utilisateur;
import com.lucsorel.hesperis.client.modele.Warning;

public class Deconnexion
{
	// lien vers l'application principale
	final Hesperis iH_hesperis;

	// widgets
	final VerticalPanel iVP_deconnexionPanel;
	final HTML iH_message;
	final Label iL_login;
	final Label iL_nom;
	final Button iB_deconnexion;

	public Deconnexion( Hesperis pH_hesperis )
	{
		// mémorisation du lien à l'application principale
		iH_hesperis = pH_hesperis;

		/*
		 *  Création du widget
		 */
		// 1. création du panneau général de la vue
		iVP_deconnexionPanel = new VerticalPanel();
		iVP_deconnexionPanel.setStylePrimaryName( "deconnexion-global" );

		// 2. panneau d'affichage de messages
		iH_message = new HTML();
		iH_message.setStylePrimaryName( "message" );

		// 3. panneau d'affichage des informations "utilisateur"
		HorizontalPanel lHP_utilisateurBoutonPanel = new HorizontalPanel();
		lHP_utilisateurBoutonPanel.setStylePrimaryName( "utilisateur-infos" );

		HorizontalPanel lHP_utilisateurPanel = new HorizontalPanel();
		lHP_utilisateurPanel.setStylePrimaryName( "utilisateur" );

		lHP_utilisateurPanel.add( new Label("Connecté(e) en tant que '") );

		// 3.1 ajout du compte sous lequel s'est connecté l'utilisateur
		iL_login = new Label();
		iL_login.setStylePrimaryName( "login" );
		lHP_utilisateurPanel.add( iL_login );

		lHP_utilisateurPanel.add( new Label("' : ") );

		// 3.2 ajout du compte sous lequel s'est connecté l'utilisateur
		iL_nom = new Label();
		iL_nom.setStylePrimaryName( "nom" );
		lHP_utilisateurPanel.add( iL_nom );
		
		// 3.3 ajout du bouton de déconnexion
		iB_deconnexion = new Button( "Déconnexion" );
		iB_deconnexion.setStylePrimaryName( "deconnexion" );
		iB_deconnexion.addClickHandler( new DeconnexionClickHandler() );

		// 4. assemblage de l'interface
		iVP_deconnexionPanel.add( iH_message );
		lHP_utilisateurBoutonPanel.add( lHP_utilisateurPanel );
		lHP_utilisateurBoutonPanel.add( iB_deconnexion );
		iVP_deconnexionPanel.add( lHP_utilisateurBoutonPanel );
	}

	public Widget asWidget()
	{
		return iVP_deconnexionPanel;
	}

	public void afficherUtilisateur()
	{
		Utilisateur lU_utilisateur = Utilisateur.Instance();
		// l'utilisateur n'est pas valide, nettoyage de l'affichage et redirection sur l'interface de connexion
		if( ! lU_utilisateur.estValide() )
		{
			// nettoyage des informations utilisateur
			iL_login.setText( "" );
			iL_nom.setText( "" );
			
			// déconnexion avec un clic de déconnexion simulé
			new DeconnexionClickHandler().onClick( null );
		}
		// l'utilisateur est valide : enrichissement de l'interface + message d'accueil
		else
		{
			iL_login.setText( lU_utilisateur.getLogin() );
			iL_nom.setText( lU_utilisateur.getNom() );
			String ls_message = "Bonjour <b>" + lU_utilisateur.getNom() + "</b>, bienvenue dans Hespéris";
			afficherMessage( new Message(ls_message) );
		}
	}

	/**
	 * Gère une demande de déconnexion à l'application :
	 * - si l'utilisateur a cliqué sur le bouton de déconnexion
	 * - demande simulée par l'application
	 */
	class DeconnexionClickHandler implements ClickHandler
	{
		@Override
		public void onClick( ClickEvent pE_clickEvent )
		{
			// réinitialisation de l'utilisateur
			Utilisateur.Instance().reinitialiser();

			/* - si le clic de déconnexion a été simulé : c'est que l'application l'a demandé (utilisateur invalide par exemple)
			 *   affichage d'un message et déconnexion dans 3s
			 * - sinon déconnexion immédiate
			 */
			if( pE_clickEvent == null )
			{
				afficherMessage( new Erreur("Problème de connexion, vous allés être redirigés vers le formulaire de connexion dans 3s...") );

				// redirection vers le formulaire de connexion après 3s
				Timer lT_timer = new Timer()
				{
					@Override
					public void run()
					{
						deconnecter();
					}
				};
				lT_timer.schedule( 3000 );
			}
			else
			{
				deconnecter();
			}
		}

		/**
		 * Envoie la requête de déconnexion au serveur (suppression de la session côté serveur)
		 */
		private void deconnecter()
		{
			// construction de la requête de déconnexion
			String ls_demandeDeconnexion = "controleur.php?action=deconnecter";
			ls_demandeDeconnexion = URL.encode(ls_demandeDeconnexion);

			// envoi de la requête avec la méthode POST
			RequestBuilder lRB_requeteDeconnexion = new RequestBuilder( RequestBuilder.POST, ls_demandeDeconnexion );
			try
			{
				lRB_requeteDeconnexion.sendRequest( null, new DeconnexionRequestCallback() );
			}
			catch( RequestException pRE_exception )
			{
				afficherMessage( new Erreur(
						"Requête de déconnexion erronnée :\n" + pRE_exception.getMessage()) );
			}
		}
	}

	/**
	 * Gère le retour de la requête de déconnexion
	 */
	class DeconnexionRequestCallback implements RequestCallback
	{
		@Override
		public void onError( Request request, Throwable pRE_exception )
		{
			afficherMessage( new Erreur(
					"Requête de connexion erronnée :\n" + pRE_exception.getMessage()) );
		}
		@Override
		public void onResponseReceived( Request request, Response pR_retourDeconnexion )
		{
			// gestion des erreurs
			String ls_texteReponse = pR_retourDeconnexion.getText();
			boolean lb_deconnexion = ls_texteReponse.equals( "true" );
			Message lM_message = null;

			// affichage d'un message d'erreur si la déconnexion a échouée
			if( ! lb_deconnexion )
			{
				if( pR_retourDeconnexion.getStatusCode() != Response.SC_OK )
				{
					lM_message = new Erreur( "Erreur : " + pR_retourDeconnexion.getStatusText()
							+ "\n Code : " + pR_retourDeconnexion.getStatusCode()
							+ "\n Header : [" + pR_retourDeconnexion.getHeadersAsString() + "]" );
				}
				// affichage du message d'erreur
				else
				{
					lM_message = new Erreur( ls_texteReponse );
				}
				afficherMessage( lM_message );
			}
			// redirection sur la page de connexion si la déconnexion a été validée
			else
			{
				iH_hesperis.afficherConnexion();
			}
		}
		
	}

	/**
	 * Affiche un message dans le panneau de message
	 * @param pM_message 
	 */
	public void afficherMessage( Message pM_message )
	{
		if( pM_message instanceof Erreur )
		{
			iH_message.setStylePrimaryName( "message-erreur" );
			iH_message.setHTML( pM_message.getMessage() );
		}
		else if( pM_message instanceof Warning )
		{
			iH_message.setStylePrimaryName( "message-warning" );
			iH_message.setHTML( pM_message.getMessage() );
		}
		// nettoyage du stylage, affichage du message
		else if( pM_message != null )
		{
			iH_message.setStylePrimaryName( "message" );
			iH_message.setHTML( pM_message.getMessage() );
		}
		// nettoyage du stylage et du message
		else
		{
			iH_message.setStylePrimaryName( "message" );
			iH_message.setHTML( "" );
		}
	}
}