/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

package com.lucsorel.hesperis.client.presview;

import com.lucsorel.hesperis.client.event.DemandeConnexionEvent;
import com.lucsorel.hesperis.client.presview.IConnexion.Display;

import org.enunes.gwt.mvp.client.EventBus;
import org.enunes.gwt.mvp.client.presenter.BasePresenter;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.inject.Inject;

public class ConnexionPresenter extends BasePresenter<Display> implements IConnexion
{
	// présenteur gérant le module d'affichage des messages
	protected final IMessage iP_message;

	@Inject
	public ConnexionPresenter( EventBus pEB_eventBus, Display pD_display,
			IMessage pP_message )
	{
		super( pEB_eventBus, pD_display );

		// récupération du panneau d'affichage des messages, enregistrements des gestionnaires et ajout à la vue
		iP_message = pP_message;
		iP_message.bind();
		display.ajouterMessageDisplay( iP_message.getDisplay() );
	}

	/**
	 * Supprime les associations de base entre la vue et les présenteurs intégrés à celui-ci
	 */
	@Override
	public void unbind()
	{
		super.unbind();

		// Suppression des gestionnaires d'événements du présenteur MessagePresenter
		if( iP_message != null )
		{
			iP_message.unbind();
		}
	}

	/**
	 * Crée les associations de base entre la vue et le présenteur
	 */
	@Override
	public void bind()
	{
		super.bind();

		/*****************************************************************************\
		 * Envoie au bus d'événements les signaux envoyés par les éléments de la vue *
		\*****************************************************************************/

		// Transforme un événement DOM de getConnexionClickHandlers() en l'événement métier DemandeConnexionEvent
		registerHandler( display.getConnexionClickHandlers().addClickHandler(
				new ClickHandler()
				{
					public void onClick( ClickEvent pE_clickEvent )
					{
						envoyerDemandeConnexionEvent();
					}
				}));

		// Transforme un événement DOM de getLoginKeyUpHandlers() en l'événement métier DemandeConnexionEvent
		// simule un clic sur le bouton de connexion si la touche [Entrée] est appuyée dans ce champ
		registerHandler( display.getLoginKeyUpHandlers().addKeyUpHandler(
				new KeyUpHandler()
				{
					public void onKeyUp( KeyUpEvent pE_keyUpEvent )
					{
						if( pE_keyUpEvent.getNativeKeyCode() == KeyCodes.KEY_ENTER )
						{
							envoyerDemandeConnexionEvent();
						}
					}
				}));
		// Transforme un événement DOM de getMotDePasseKeyUpHandlers() en l'événement métier DemandeConnexionEvent
		// simule un clic sur le bouton de connexion si la touche [Entrée] est appuyée dans ce champ
		registerHandler( display.getMotDePasseKeyUpHandlers().addKeyUpHandler(
				new KeyUpHandler()
				{
					public void onKeyUp( KeyUpEvent pE_keyUpEvent )
					{
						if( pE_keyUpEvent.getNativeKeyCode() == KeyCodes.KEY_ENTER )
						{
							envoyerDemandeConnexionEvent();
						}
					}
				}));
	}

	/**
	 * Envoie au bus d'événements une demande de connexion contenant le login et mot de passe rentrés
	 */
	private void envoyerDemandeConnexionEvent()
	{
		eventBus.fireEvent( 
				new DemandeConnexionEvent( 
						display.getLoginTexte().getText(),
						display.getMotDePasseTexte().getText() )
		);
	}
}