/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

package com.lucsorel.hesperis.client.presview;

import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.VerticalPanel;

import com.lucsorel.hesperis.client.presview.IDeconnexion.Display;

public class DeconnexionDisplay extends Composite implements Display
{
	/**
	 * Déclaration des éléments graphiques composant la vue, et qui pourront être renvoyés 
	 * par le widget sous forme de ...HasHandlers. Ces méthodes typées ...HasHandler doivent
	 * être déclarées dans l'interface IDeconnexion associée.
	 */
	// Panneau englobant le widget (permet une initialisation du widget dans le constructeur)
	private VerticalPanel iVP_deconnexionPanel;
	final SimplePanel iSP_message;
	final Label iL_login;
	final Label iL_nom;
	final Button iB_deconnexion;
	
	/**
	 * Le module GIN a besoin d'un constructeur sans paramètre 
	 * pour pouvoir faire de l'injection de dépendances.
	 */
	public DeconnexionDisplay()
	{
		/*
		 *  Création du widget
		 */
		// 1. création du panneau général de la vue
		iVP_deconnexionPanel = new VerticalPanel();
		iVP_deconnexionPanel.setStylePrimaryName( "deconnexion-global" );

		// 2. accueil du panneau d'affichage de messages
		iSP_message = new SimplePanel();

		// 3. panneau d'affichage des informations "utilisateur"
		HorizontalPanel lHP_utilisateurBoutonPanel = new HorizontalPanel();
		lHP_utilisateurBoutonPanel.setStylePrimaryName( "utilisateur-infos" );

		HorizontalPanel lHP_utilisateurPanel = new HorizontalPanel();
		lHP_utilisateurPanel.setStylePrimaryName( "utilisateur" );

		lHP_utilisateurPanel.add( new Label("Connecté(e) en tant que '") );

		// 3.1 ajout du compte sous lequel s'est connecté l'utilisateur
		iL_login = new Label();
		iL_login.setStylePrimaryName( "login" );
		lHP_utilisateurPanel.add( iL_login );

		lHP_utilisateurPanel.add( new Label("' : ") );

		// 3.2 ajout du compte sous lequel s'est connecté l'utilisateur
		iL_nom = new Label();
		iL_nom.setStylePrimaryName( "nom" );
		lHP_utilisateurPanel.add( iL_nom );
		
		// 3.3 ajout du bouton de déconnexion
		iB_deconnexion = new Button( "Déconnexion" );
		iB_deconnexion.setStylePrimaryName( "deconnexion" );

		// 4. assemblage de l'interface
		iVP_deconnexionPanel.add( iSP_message );
		lHP_utilisateurBoutonPanel.add( lHP_utilisateurPanel );
		lHP_utilisateurBoutonPanel.add( iB_deconnexion );
		iVP_deconnexionPanel.add( lHP_utilisateurBoutonPanel );
		
		// définit "iVP_deconnexionPanel" comme étant le composant graphique représentant cette classe
		initWidget( iVP_deconnexionPanel );
	}
	
	/**
	 * Méthode surchargée pour l'implémentation du pattern MVP
	 * @see org.enunes.gwt.mvp.client.view.Display#asWidget()
	 */
	@Override
	public Widget asWidget()
	{
		return this;
	}

	/**
	 * Ajoute la vue du module d'affichage des messages dans le widget général de déconnexion
	 * @see com.lucsorel.hesperis.client.presview.IDeconnexion.Display#ajouterMessageDisplay(IMessage.Display)
	 */
	@Override
	public void ajouterMessageDisplay( IMessage.Display display )
	{
		this.iSP_message.setWidget( display.asWidget() );
	}

	/**
	 * @see com.lucsorel.hesperis.client.presview.IDeconnexion.Display#getLoginTexte()
	 */
	@Override
	public HasText getLoginTexte()
	{
		return this.iL_login;
	}

	/**
	 * @see com.lucsorel.hesperis.client.presview.IDeconnexion.Display#getNomTexte()
	 */
	@Override
	public HasText getNomTexte()
	{
		return this.iL_nom;
	}

	/**
	 * Renvoie le bouton de déconnexion à travers l'interface de gestion de clic
	 * @see com.lucsorel.hesperis.client.presview.IDeconnexion.Display#getDeconnexionClickHandlers()
	 */
	@Override
	public HasClickHandlers getDeconnexionClickHandlers()
	{
		return this.iB_deconnexion;
	}
}