/**
 * Moi (Luc Sorel, http://www.lucsorel.com/index.php?page=contact) reconnaît être l'auteur de ce code source. 
 * J'ai placé ce code source dans le domaine publique. Vous pouvez utiliser, modifier et distribuer le code 
 * source et les programmes exécutables basés sur ce code source. Veuillez toutefois prendre en considération 
 * les points suivants :
 * 
 * --- Décharge légale de garantie ---
 * Ce code source vous est présenté "en tant que tel" et sans garantie de performance ou d'intérêt commercial. 
 * L'auteur et / ou le distributeur ont pu faire des déclarations ) propos de ce code source. Ces déclarations 
 * n'ont pas valeur de garanties et ne doivent pas servir de fondements quant à l'utilisation tierce de ce code 
 * source.
 * 
 * Ce code source est mis à disposition sans aucune garantie implicite ou explicite. Étant donné la diversité 
 * des conditions contextuelles et matérielles sous lesquelles ce code source peut être utilisé, aucune garantie 
 * d'adéquation pour un usage spécifique n'est proposé. il est conseillé que l'utilisateur teste le code source 
 * consciencieusement avant de s'appuyer dessus. L'utilisateur prend l'entière responsabilité de l'utilisation 
 * de ce code source.
 */

package com.lucsorel.hesperis.client.presview;

import com.lucsorel.hesperis.client.ajax.DeconnexionRequestCallback;
import com.lucsorel.hesperis.client.ajax.ConnexionRequestCallback;
import com.lucsorel.hesperis.client.event.AfficherConnexionEvent;
import com.lucsorel.hesperis.client.event.AfficherConnexionHandler;
import com.lucsorel.hesperis.client.event.AfficherDeconnexionEvent;
import com.lucsorel.hesperis.client.event.AfficherDeconnexionHandler;
import com.lucsorel.hesperis.client.event.DemandeConnexionEvent;
import com.lucsorel.hesperis.client.event.DemandeConnexionHandler;
import com.lucsorel.hesperis.client.event.DemandeDeconnexionEvent;
import com.lucsorel.hesperis.client.event.DemandeDeconnexionHandler;
import com.lucsorel.hesperis.client.event.MessageEvent;
import com.lucsorel.hesperis.client.modele.Erreur;
import com.lucsorel.hesperis.client.modele.Utilisateur;
import com.lucsorel.hesperis.client.presview.IHesperisMvp.Display;

import org.enunes.gwt.mvp.client.EventBus;
import org.enunes.gwt.mvp.client.presenter.BasePresenter;
import org.enunes.gwt.mvp.client.presenter.Presenter;

import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestException;
import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.Timer;
import com.google.inject.Inject;
import com.google.inject.Provider;

public class HesperisMvpPresenter extends BasePresenter<Display> implements IHesperisMvp
{
	private final Provider<IConnexion> iPV_connexionFournisseur;
	private final Provider<IDeconnexion> iPV_deconnexionFournisseur;
	private Presenter<? extends org.enunes.gwt.mvp.client.view.Display> iP_connexionDeconnexion;

	@Inject
	public HesperisMvpPresenter( 
			EventBus pEB_eventBus, 
			Display pD_display,
			Provider<IConnexion> pPV_connexionFournisseur,
			Provider<IDeconnexion> pPV_deconnexionFournisseur
			)
	{
		super( pEB_eventBus, pD_display );
		
		// récupération des fournisseurs de Présenteurs
		this.iPV_connexionFournisseur = pPV_connexionFournisseur;
		this.iPV_deconnexionFournisseur = pPV_deconnexionFournisseur;

		// affichage de la vue de connexion
		switchPresenterConnexionDeconnexion( this.iPV_connexionFournisseur.get() );
	}

	/**
	 * Créée les gestionnaires d'événement propres au Présenteur HesperisMvpPresenter
	 */
	@Override
	public void bind()
	{
		super.bind();

		/******************************************************************\
		 * Traitement des événements "métier" envoyés au bus d'événements *
		\******************************************************************/
		// Capte les événements de type DemandeConnexionEvent pour lancer la méthode de traitement
		registerHandler( eventBus.addHandler(DemandeConnexionEvent.getType(),
				new DemandeConnexionHandler() {
					@Override
					public void onDemandeConnexion(DemandeConnexionEvent pE_DemandeConnexion)
					{
						doDemanderConnexion( pE_DemandeConnexion );
					}
				}) );
		// Capte les événements de type AfficherDeconnexionEvent pour lancer la méthode de traitement
		registerHandler( eventBus.addHandler(AfficherDeconnexionEvent.getType(),
				new AfficherDeconnexionHandler() {
					@Override
					public void onAfficherDeconnexion(AfficherDeconnexionEvent pE_AfficherDeconnexion)
					{
						doAfficherDeconnexion( pE_AfficherDeconnexion );
					}
				}) );

		// Capte les événements de type DemandeDeconnexionEvent pour lancer la méthode de traitement
		registerHandler( eventBus.addHandler(DemandeDeconnexionEvent.getType(),
				new DemandeDeconnexionHandler() {
					@Override
					public void onDemandeDeconnexion(DemandeDeconnexionEvent pE_DemandeDeconnexion)
					{
						gererDemanderDeconnexion( pE_DemandeDeconnexion );
					}
				}) );
		// Capte les événements de type AfficherConnexionEvent pour lancer la méthode de traitement
		registerHandler( eventBus.addHandler(AfficherConnexionEvent.getType(),
				new AfficherConnexionHandler() {
					@Override
					public void onAfficherConnexion(AfficherConnexionEvent pE_AfficherConnexion)
					{
						doAfficherConnexion( pE_AfficherConnexion );
					}
				}) );
	}

	/**
	 * @param pEAfficherConnexion
	 */
	protected void doAfficherConnexion( AfficherConnexionEvent pEAfficherConnexion )
	{
		// récupération du module de déconnexion
		IConnexion lP_connexion = this.iPV_connexionFournisseur.get();

		// affichage de la vue de déconnexion
		switchPresenterConnexionDeconnexion( lP_connexion );
	}

	/**
	 * Gère le time out avant de déléguer la demande de déconnexion à doDemanderDeconnexion()
	 * @param pE_demandeDeconnexion
	 */
	protected void gererDemanderDeconnexion( DemandeDeconnexionEvent pE_demandeDeconnexion )
	{
		// réinitialisation de l'utilisateur
		Utilisateur.Instance().reinitialiser();
		int li_timeOut = pE_demandeDeconnexion.getTimeOutMilliSecondes();
		// déconnexion délayée
		if( li_timeOut > 0 )
		{
			eventBus.fireEvent( new MessageEvent(
					new Erreur("Problème de connexion, vous allés être redirigés vers le formulaire de connexion dans " 
							+ (li_timeOut/1000) + "s...")
			) );
			Timer lT_timer = new Timer()
			{
				@Override
				public void run()
				{
					doDemanderDeconnexion();
				}
			};
			lT_timer.schedule( 3000 );

		}
		// déconnexion directe
		else
		{
			doDemanderDeconnexion();
		}
	}

	/**
	 * Demande la déconnexion de l'utilisateur au serveur
	 */
	private void doDemanderDeconnexion()
	{
		// construction de la requête de déconnexion
		String ls_demandeDeconnexion = "controleur.php?action=deconnecter";
		ls_demandeDeconnexion = URL.encode(ls_demandeDeconnexion);

		// envoi de la requête avec la méthode POST
		RequestBuilder lRB_requeteDeconnexion = new RequestBuilder( RequestBuilder.POST, ls_demandeDeconnexion );
		try
		{
			lRB_requeteDeconnexion.sendRequest( null, DeconnexionRequestCallback.Instance(eventBus) );
		}
		catch( RequestException pRE_exception )
		{
			eventBus.fireEvent( new MessageEvent(
					new Erreur("Requête de connexion erronnée :\n" + pRE_exception.getMessage())
			) );
		}
	}

	/**
	 * @param pEAfficherDeconnexion
	 */
	protected void doAfficherDeconnexion( AfficherDeconnexionEvent pEAfficherDeconnexion )
	{
		// récupération du module de déconnexion
		IDeconnexion lP_deconnexion = this.iPV_deconnexionFournisseur.get();

		// affichage de la vue de déconnexion
		switchPresenterConnexionDeconnexion( lP_deconnexion );

		// mise à jour de la vue avec les informations demandées
		if( pEAfficherDeconnexion.doitAfficherUtilisateur() )
		{
			lP_deconnexion.afficherUtilisateur();
		}
	}

	/**
	 * Demande une connexion d'utilisateur au serveur
	 * @param pE_demandeConnexion
	 */
	protected void doDemanderConnexion( DemandeConnexionEvent pE_demandeConnexion )
	{
		String ls_demandeConnexion = "controleur.php?"
			+ "action=connecter&login=" + pE_demandeConnexion.getLogin()
			+ "&password=" + pE_demandeConnexion.getMotDePasse();
		ls_demandeConnexion = URL.encode(ls_demandeConnexion);

		// envoi de la requête avec la méthode POST
		RequestBuilder lRB_requeteDemandeConnexion = new RequestBuilder( RequestBuilder.POST, ls_demandeConnexion );
		try
		{
			lRB_requeteDemandeConnexion.sendRequest( null, ConnexionRequestCallback.Instance(eventBus) );
		}
		catch( RequestException pRE_exception )
		{
			eventBus.fireEvent( new MessageEvent(
					new Erreur("Requête de connexion erronnée :\n" + pRE_exception.getMessage())
			) );
		}
	}

	/**
	 * Affiche un présenteur de connexion ou de déconnexion dans ce présenteur en prenant
	 * soin de libérer les gestionnaires d'événements du précédent présenteur affiché
	 * 
	 * @param pP_connexionDeconnexion
	 */
	private void switchPresenterConnexionDeconnexion( Presenter<? extends org.enunes.gwt.mvp.client.view.Display> pP_connexionDeconnexion )
	{
		// suppression des gestionnaires d'événement du présenteur affiché précédemment et suppression dans la vue
		if (this.iP_connexionDeconnexion != null)
		{
			this.iP_connexionDeconnexion.unbind();
			this.display.viderConnexionDeconnexion();
		}
		// nouvelle affectation de présenteur de connexion / déconnexion et affichage
		this.iP_connexionDeconnexion = pP_connexionDeconnexion;
		if (pP_connexionDeconnexion != null)
		{
			this.display.afficherConnexionDeconnexion( pP_connexionDeconnexion.getDisplay() );
			this.iP_connexionDeconnexion.bind();
		}
	}

}